# -*- coding: utf-8 -*-

"""Flask website application module."""
from flask import Flask, render_template
from website_monitor import db_utils
from datetime import datetime

app = Flask(__name__)


@app.route('/', methods=['GET'])
def index():
    records = db_utils.get_all_records()
    records = sorted(records, key=lambda x: datetime.strptime(
        x[3], "%d-%m-%Y %H:%M:%S"), reverse=True)

    return render_template('index.html', records=records)


def main():
    app.run()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""Exceptions module."""


class ConfigFileEmpty(Exception):
    """The config file is empty."""
    pass


class ConfigFileInvalid(Exception):
    """The config file is not valid."""
    pass


class URLPropertyNotFound(Exception):
    """Website property does not specify required URL"""


class RequirementsNotFulfilled(Exception):
    """Content requirements were not matched for specific page response."""
    pass



# -*- coding: utf-8 -*-

"""Database utils module."""

import os
import sqlite3

DB_NAME = 'website_monitor.db'
DB_TABLE_NAME = 'website_checks'


def get_connection():
    """
    Helper function used to initiate database connection.

    :return: Tuple of two items - sqlite3.Connection class instance
        and sqlite3.Cursor class instance
    """
    conn, cur = None, None
    path = os.path.join(os.path.dirname(os.path.abspath(__file__)), DB_NAME)

    try:
        conn = sqlite3.connect(path)
    except Exception as e:
        print("Error while making connection with DB: {}".format(e))

    if conn is not None:
        cur = conn.cursor()
    return (conn, cur)


def create_table():
    """
    Helper function used to populate database with nescessary table.

    :return: None
    """
    conn, cur = get_connection()
    try:
        cur.execute(
            '''
            CREATE TABLE website_checks (
                id	INTEGER,
                webname	TEXT,
                url	TEXT,
                request_time	REAL,
                status	INTEGER,
                response_time	REAL,
                requirements	INTEGER,
                error	TEXT,
                PRIMARY KEY(id))
            '''
        )
        conn.commit()
    except Exception as e:
        print("Error while creating table: {}".format(e))
    finally:
        conn.close()


def record_insert(webname, url, request_time=None, status=None,
                  response_time=None, requirements=None, error=None):
    """
    Helper function used to create records in the database table.

    :param webname: Alias name of the website
    :param url: Website URL
    :param request_time: Time at which request was made
    :param status: Status of HTTP response to specific website
    :param response_time: Time to complete whole request
    :param requirements: Content requirements for specific website
    :param error: Error messages for specific website
    :return: None
    """
    conn, cur = get_connection()
    request_time = request_time.strftime("%d-%m-%Y %H:%M:%S")
    try:
        cur.execute(
            '''
            INSERT INTO website_checks (
                webname, url, request_time, status, response_time,
                requirements,error
            ) VALUES(?,?,?,?,?,?,?)
            ''', (webname, url, request_time, status, response_time,
                  requirements, error))
        conn.commit()
    except Exception as e:
        print("Error while making INSERT: {}".format(e))
    finally:
        conn.close()


def get_all_records():
    """
    Helper function used to fetch all the records from database table.

    :return: List of tuples representing status check records.
    """
    conn, cur = get_connection()
    records = []
    try:
        cur.execute('''SELECT * FROM website_checks''')
        records = cur.fetchall()
    except Exception as e:
        print("Error while getting all records: {}".format(e))
    finally:
        conn.close()

    return records



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `website_monitor` package."""

import pytest
import json

from website_monitor.website_monitor import Config, Monitor
from website_monitor.wm_exceptions import (
    ConfigFileEmpty, ConfigFileInvalid, RequirementsNotFulfilled,
    URLPropertyNotFound
)


class TestConfig(object):
    @pytest.fixture
    def config_path(self, tmpdir):
        return str(tmpdir / 'config_test.json')

    def test_initialization_with_default_values_works_correctly(self):
        config = Config()

        assert hasattr(config, 'check_period')
        assert hasattr(config, 'websites')

    def test_initialization_with_not_empty_config_file_initializes_instance_variables(
            self):
        config = Config()

        assert config.check_period > 0
        assert len(config.websites) > 0

    def test_initialization_with_empty_config_file_should_fail(
            self, config_path):
        with open(config_path, 'w') as f:
            json.dump({}, f)

        with pytest.raises(ConfigFileEmpty):
            Config(config_abs_path=config_path)

    def test_initialization_with_not_empty_config_file_without_website_properties_should_fail(
            self, config_path):
        with open(config_path, 'w') as f:
            json.dump({'check_period': 60}, f)

        with pytest.raises(ConfigFileInvalid):
            Config(config_abs_path=config_path)

    def test_initialization_with_not_empty_config_file_without_website_url_property_should_fail(
            self, config_path):
        with open(config_path, 'w') as f:
            json.dump({'google_index': {"content": "Search"}}, f)

        with pytest.raises(URLPropertyNotFound):
            Config(config_abs_path=config_path)

    def test_setting_check_period_value_to_negative_should_fail(self):
        with pytest.raises(ValueError):
            Config(check_period=-1)


class TestMonitor(object):
    url_valid = 'https://www.google.pl'
    url_invalid = 'https://oainsdobfuq2.com'

    @pytest.fixture
    def config(self):
        return Config()

    @pytest.fixture
    def monitor(self, config):
        return Monitor(config)

    @pytest.fixture
    def response_valid(self):
        return Monitor.make_request(self.__class__.url_valid)

    def test_make_request_with_valid_url_should_return_response_object(
            self, response_valid):
        response = response_valid
        assert response is not None

    def test_make_request_with_valid_url_should_return_200_status_code(
            self, response_valid):
        response = response_valid
        assert response.status_code == 200

    def test_make_request_with_invalid_url_should_return_none(self):
        response = Monitor.make_request(self.__class__.url_invalid)
        assert response is None

    def test_check_requirements_should_return_true_if_requirements_are_met(
            self, response_valid):
        response = response_valid
        meets_requirements = Monitor.check_requirements(response, 'google')
        assert meets_requirements is True

    def test_check_requirements_should_raise_exception_if_requirements_are_met(
            self, response_valid):
        response = response_valid
        with pytest.raises(RequirementsNotFulfilled):
            Monitor.check_requirements(response, 'thistextisnotonthepage')



# -*- coding: utf-8 -*-

"""Main module."""
import copy
import datetime
import getopt
import json
import logging as log
import os
import re
import sys
import threading
import time

import requests

from website_monitor import db_utils
from .wm_exceptions import (
    ConfigFileEmpty, ConfigFileInvalid, RequirementsNotFulfilled,
    URLPropertyNotFound
)

WORK_DIR = os.path.dirname(os.path.abspath(__file__))
LOG_DIR = os.path.join(WORK_DIR, 'logs')
LOG_FILE_PATH = os.path.join(LOG_DIR, 'logfile')

log.basicConfig(filename=LOG_FILE_PATH, format='%(message)s', level=log.INFO)


class Config(object):
    """Represents a configuration object."""

    def __init__(self, config_abs_path=None, check_period=0):
        """
        Initialize a Config instance.

        :param config_abs_path: String representing absolute path
            to configuration file
        :param check_period: Value representing the interval period between
            website status checks.
        """
        config_path = config_abs_path or os.path.join(WORK_DIR, 'config.json')
        configs = self._load_config(config_path)
        if check_period:
            # if check_period was set on CLI then it takes priority
            # over one specified in config file
            configs.pop('check_period', None)

        # sets default check_period to 3600s(1h) if not specified on CLI
        # nor in config file.
        self.check_period = check_period or configs.pop('check_period', 3600)
        self.websites = configs

    @property
    def check_period(self):
        return self.__check_period

    @check_period.setter
    def check_period(self, val):
        try:
            val = int(val)
        except ValueError:
            s = ('Please make sure that check period value is specified '
                 'as integer.')
            raise ValueError(s)

        if val < 0:
            s = ('Checking period cannot be negative. Please set correct '
                 'value and try again.')
            raise ValueError(s)
        self.__check_period = val

    @staticmethod
    def _validate_config(configs):
        """
        Class method used for validating config file.

        :param configs: dict containing configuration parameters
        :return: If successfully validated returns True,
            otherwise raises Exception
        """

        # check if config file is completely empty
        if not len(configs):
            s = ('The config file is empty. Please ensure that config file '
                 'is not empty and try again.')
            raise ConfigFileEmpty(s)

        # check if there are any other properties except 'check_period'
        tmp_data = copy.copy(configs)
        tmp_data.pop('check_period', 0)
        if not len(tmp_data):
            s = ('Please enter settings for at least one website in your '
                 'config file and try again.')
            raise ConfigFileInvalid(s)

        # check if website proprties have at least defined url
        for webname, web_data in configs.items():
            if webname == 'check_period':
                continue
            if 'url' not in web_data:
                s = ('Website property {} does not contain required URL '
                     'property. Please specify URL property and try again.')
                raise URLPropertyNotFound(s.format(webname))
        return True

    def _load_config(self, config_path):
        """
        Helper function used to read settings from configuration file.

        :return: If successfully read config file returns True
        """
        with open(config_path) as f:
            configs = json.load(f)

        self.__class__._validate_config(configs)

        return configs


class Monitor(object):
    """Represents Monitor object."""
    config_obj = None

    def __init__(self, config_obj):
        """
        Initialize a Monitor instance.

        :param config_obj: website_monitor.Config class instance
        """
        self.config = config_obj
        self.next_call = time.time()

    def start_watch(self):
        """
        Method responsible for triggering periodic checks in time intervals.
        If time interval is not specified it is set by default to 3600s(1h).

        :return: None
        """
        self._start_checks()
        self.next_call += self.config.check_period
        # accounts for drift
        # more at https://stackoverflow.com/a/18180189/2808371
        threading.Timer(self.next_call - time.time(), self.start_watch).start()

    def _start_checks(self):
        """
        Method responsible for coordinating checks of each website.

        :return: None
        """
        # used for formatting first and last message of round of checks
        time_format = '%d/%m/%Y %H:%M:%S'
        asterix = '*' * 10
        s = ('\n{asterix}Starting new round of checks - {current_time}'
             '{asterix}')
        log.info(s.format(asterix=asterix,
                          current_time=datetime.datetime.now().strftime(
                              time_format)))

        threads = []
        for webname, web_data in self.config.websites.items():
            url = web_data['url']
            content_requirements = web_data.get('content', None)
            t = threading.Thread(target=self._perform_checks, args=(
                url, content_requirements, webname))
            threads.append(t)
            t.start()

        for t in threads:
            t.join()
        s = '\n{asterix}Finished all checks - {current_time}{asterix}'
        log.info(s.format(asterix=asterix,
                          current_time=datetime.datetime.now().strftime(
                              time_format)))

    def _perform_checks(self, url, content_requirements, webname):
        """
        Method responsible for checking requirements on each website.

        :param url: URL of the page for which we want to check requirements
        :param content_requirements: Actual content requirements
        :return: None
        """
        response = self.make_request(url, webname)

        if not response:
            return
        response_time = response.elapsed / datetime.timedelta(seconds=1)
        try:
            self.check_requirements(response, content_requirements)
        except RequirementsNotFulfilled as e:
            s = ('Content requirements: {e} ("{content_requirements}" '
                 'not in response content)')
            log.info(s.format(**locals()))
            db_utils.record_insert(webname, url, datetime.datetime.now(),
                                   response.status_code, response_time, 0)
        else:
            s = ('Content requirements: Website meets content requirements.'
                 '("{content_requirements}" in response content)')
            log.info(s.format(**locals()))
            db_utils.record_insert(webname, url, datetime.datetime.now(),
                                   response.status_code, response_time, 1)

    @staticmethod
    def make_request(url, webname=None):
        """
        Static method used to perform actual request to the server.

        :param url: URL of the page that we want to make request to
        :param webname: Alias name for website
        :return: If successful returns requests.Response object, otherwise None
        """
        try:
            response = requests.get(url)
        except requests.exceptions.RequestException as e:
            error_msg = str(e)
            s = 'Connection problem\nError message: {}\n'
            log.info(s.format(error_msg))
            db_utils.record_insert(
                webname, url, request_time=datetime.datetime.now(),
                error=error_msg)
        else:
            s = ('\nURL: {url}\nStatus: {response.status_code}\n'
                 'Response time: {response.elapsed.seconds}s'
                 '{response.elapsed.microseconds}\u00B5s')
            log.info(s.format(**locals()))
            return response
        return None

    @staticmethod
    def check_requirements(response, content_requirements):
        """
        Static method used to perform requirement checks for specific
        requests.Response.

        :param response: requests.Response object.
        :param content_requirements: Content requirements to check against
            in response object.
        :return: If requirements are met returns True, otherwise raises
            website_monitor.exceptions.RequirementsNotFulfilled
        """
        response_content = response.content.decode('utf-8', 'ignore')
        requirements_are_met = re.search(content_requirements,
                                         response_content, re.IGNORECASE)

        if not content_requirements or requirements_are_met:
            # if there are no requirements or the requirements are fulfilled
            return True
        s = 'Website content does not match specified requirements.'
        raise RequirementsNotFulfilled(s.format(**locals()))


def parse_cl_args(argv):
    """
    Helper function used to check if user provided checking period value
    in command line arguments.

    :param argv: command line arguments
    :return: checking period value
    """
    help_text = """
    Usage:
        website_monitor.py -i <checking_interval_in_s>
        website_monitor.py --interval=<checking_interval_in_s>
    """
    try:
        opts, args = getopt.getopt(argv, "hi:", ["help", "interval="])
    except getopt.GetoptError:
        print(help_text)
        sys.exit(2)
    for opt, val in opts:
        if opt == '-h':
            print(help_text)
            sys.exit(0)
        elif opt in ("-i", "--interval"):
            return val

def main():
    interval = parse_cl_args(sys.argv[1:])
    db_utils.create_table()
    config = Config(check_period=interval)
    Monitor(config).start_watch()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Flask==0.12.2', 'requests==2.18.4']

setup_requirements = ['pytest-runner']

test_requirements = ['pytest']

setup(
    author="Damian P.",
    author_email='an0o0nyme@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    description=("Website Monitor is a program used for monitoring web sites "
                 "and reporting their availability."),
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='website_monitor',
    name='website_monitor',
    packages=find_packages(include=['website_monitor', 'web_app']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/an0o0nym/website_monitor',
    version='0.1.2',
    zip_safe=False,
    entry_points = {
        'console_scripts': [
            'website_monitor=website_monitor.website_monitor:main',
            'website_monitor_web_app=web_app.app:main'
        ],
    }
)
